//Action Engine
//Version 1.00 (Apr 20, 2020)

var actionTimer;
var actionStep = -1;

var firstBkg = true;
var firstWrongBkg = true;
var firstMove = true;
var firstRotate = 0;

var firstIntoStep2 = true;

var previousShape = null;
var currentShape = null;
var nextShape = null;

var iCurrentShape = 0;
var side = 0;
var isClose = false;
var isRightAngle = false;
var systemSound = '';
var sndIndex = 0;

var AE_DOIT = 1;	//do it and continue
var AE_SKIP = 0;	//skip it and continue
var AE_EXIT = -1;	//do it and return to interactive mode
var AE_DROP = -2;	//return to interactive mode immediately

var actionAudio;

function playVoice() {
    actionAudio.pause();
    if (systemSound != '') {
        actionAudio.src = systemSound;
    } else if (actionList[actionStep].sound != undefined) {
        //different shapes can have different sounds 
        var audioSrc = actionList[actionStep].sound.split(",");
        if (audioSrc.length == 1) {
            actionAudio.src = audioSrc[0];
        } else {
            actionAudio.src = audioSrc[sndIndex];
        }
    } else {
        return;
    }
    actionAudio.load();
    actionAudio.loop = false;
    playPromise = actionAudio.play();
    /*	if (playPromise) {
            playPromise.then(() => {
                console.log("play sound: ", actionAudio.src);
            }).catch((e) => {
                console.log("play sound failed: ", actionAudio.src);
                if(debug){
                    document.getElementById('debug').innerHTML = e + '<br />';
                }
            });
        }*/
}

function endVoice() {
    if (actionList[actionStep].type == 'narrate') {
        document.getElementById("fullmask").style.visibility = "hidden";
        document.getElementById("highlightBox").style.visibility = "hidden";
        timer1 = setInterval(displayPoints, 100);
        timer4 = setTimeout("document.getElementById('bubble').style.visibility = 'hidden'", 1000);
        startCountdownClock(0);
    }

    jump();
}

function jump() {
    if (actionList[actionStep].delay == 0 || actionList[actionStep].delay == undefined) {
        doNextAction();
    } else if (actionList[actionStep].delay > 0) {
        actionTimer = setTimeout(doNextAction, actionList[actionStep].delay);
    } else if (actionList[actionStep].delay == -1) {
        postAction(actionList[actionStep].type);
    }
}

function doNextAction() {
    if (actionStep < actionList.length) {
        doAction(actionStep + 1);
    }
}

function doAction(i) {
    if (i < 0 || i >= actionList.length) {
        return;
    }
    actionStep = i;

    var res = AE_DOIT;
    sndIndex = 0;
    systemSound = '';
    if (actionList[i].check != undefined) {
        res = checkCondition(actionList[i].check);
        if (res == AE_DROP) {
            postAction(actionList[i].type);
            return;
        }
    }

    if (res == AE_DOIT || res == AE_EXIT) {
        switch (actionList[i].type) {
            //guide events
            case 'narrate':
                document.getElementById("fullmask").style.visibility = "visible";
                var highlightBox = document.getElementById("highlightBox");
                if (actionList[i].box != undefined) {
                    highlightBox.style.visibility = "visible";
                    highlightBox.style.top = actionList[i].box.top;
                    highlightBox.style.left = actionList[i].box.left;
                    highlightBox.style.bottom = actionList[i].box.bottom;
                    highlightBox.style.right = actionList[i].box.right;
                    highlightBox.style.width = actionList[i].box.width;
                    highlightBox.style.height = actionList[i].box.height;
                } else {
                    highlightBox.style.visibility = "hidden";
                }
                clearInterval(timer1);		//suspend score timer
                clearTimeout(timer4);		//suspend bubble timer
                startCountdownClock(-1);	//suspend time countdown timer

                actionAudio.addEventListener('ended', endVoice, false);
                playVoice(i);
                return;
            case 'talk':
                if (res == AE_DOIT) {
                    actionAudio.addEventListener('ended', endVoice, false);
                } else {
                    actionAudio.removeEventListener('ended', endVoice);
                }
                playVoice(i);
                return;
            case 'startNetWarning':
                displayBadnet();
                break;
            case 'endNetWarning':
                hideBadnet();
                break;

            //simulated server response
            case 'GetInitInfo':
                onCountdown(actionList[i].msg);
                return;
            case 'Move':
            case 'Zero':
                onFinish('Move', actionList[i].msg);
                break;
            case 'ChgBG':
                onFinish('ChgBG', actionList[i].msg);
                break;

            //simulated peer events
            case 'move':
                onMove(actionList[i].msg);
                break;
            case 'chgBG':
                onChgBkg(actionList[i].msg);
                break;
            case 'look':
                onPeerLook();
                break;
        }
    }

    if (res == AE_EXIT) {
        postAction(actionList[i].type);
        return;
    } else if (res == AE_SKIP && actionList[i].delay != -1) {
        doNextAction();
        return;
    }

    jump();
}

//Global used: currentMsg, currentAction
function checkCondition(condition) {
    switch (condition) {
        case 'firstBkg':
            if (firstBkg) {
                firstBkg = false;
                return AE_DOIT;
            }
            return AE_SKIP;

        case 'firstWrongBkg':
            if (firstWrongBkg) {
                firstWrongBkg = false;
                return AE_DOIT;
            }
            return AE_SKIP;

        case 'firstMove':
            if (firstMove == true && currentMsg.stepscore > 0 && currentAction != 0) {
                firstMove = false;
                return AE_DOIT;
            }
            return AE_SKIP;
        case 'firstRotate':
            if (currentShape != null) {
                if (currentAction == 2) {
                    firstRotate = 5;
                }
                else if (firstRotate > 0) {
                    firstRotate--;
                }
                if (firstRotate == 0 && !isRightAngle) {
                    firstRotate = 5;
                    currentShape = null;
                    return AE_EXIT;
                }
                //firstRotate--;
            }
            return AE_SKIP;

        case 'withPrev':
            if (previousShape != null && currentShape != null) {
                if (currentShape.inPlace != true && previousShape.inPlace != true) {
                    currentShape = null;
                    return AE_EXIT;
                }
            }
            return AE_SKIP;

        case 'withNext':
            if (nextShape != null && currentShape != null) {
                if (currentShape.inPlace == true && nextShape.inPace != true) {
                    currentShape = null;
                    return AE_EXIT;
                }
            }
            return AE_SKIP;

        case 'withNextStep':
            if (currentShape != null) {
                if (currentShape.inPlace != true && isRightAngle
                    && currentShape.steps > 1 && firstIntoStep2) {
                    //currentShape = null;
                    firstIntoStep2 = false;
                    return AE_EXIT;
                }
            }
            return AE_SKIP;

        case 'angle':
            if (currentShape != null) {
                if (!isRightAngle) {
                    sndIndex = iCurrentShape;
                    return AE_EXIT;
                }
            }
            return AE_SKIP;

        case 'position':
            if (currentShape != null && !isClose) {
                sndIndex = iCurrentShape;
                return AE_EXIT;
            }
            return AE_SKIP;

        case 'close':
            if (currentShape != null && isClose) {
                switch (side) {
                    case 0:
                        systemSound = 'sound/goDown.mp3';
                        break;
                    case 1:
                        systemSound = 'sound/goLeft.mp3';
                        break;
                    case 2:
                        systemSound = 'sound/goUp.mp3';
                        break;
                    case 3:
                        systemSound = 'sound/goRight.mp3';
                        break;
                }
                return AE_EXIT;
            }
            return AE_SKIP;
    }

    return AE_DOIT;
}

//only called when returning to the interactive mode
function postAction(actionType) {
}

//***********************************************************
// Following are API functions called by the game
//***********************************************************

//global used: role, params
function AE_init() {
    actionList = new Array();
    for (var i in actions) {
        actionList[i] = eval("(" + actions[i] + ")");
    }

    actionAudio = document.getElementById("snd_tutorial");
}

function AE_start() {
    doAction(0);
}

function AE_timeout() {
    actionList[timeoutAction].msg.score = 0;	//to avoid the success sound
    doAction(timeoutAction);
}

function AE_onChangeBackgroundButton() {
    clearTimeout(actionTimer);
    actionAudio.removeEventListener('ended', endVoice);
    FirstBkgBtn = false;
}

//currentMsg: globle variable  
function AE_onMove(msg) {
    currentShape = null;
    iCurrentShape = 0;
    isClose = false;

    for (var i in currentMsg.targetshapes2) {
        if (currentMsg.targetshapes2[i].shapeid == msg.id) {
            currentShape = currentMsg.targetshapes2[i];
            iCurrentShape = i;
            break;
        }
    }
    previousShape = null;
    if (currentShape.prev != undefined) {
        for (var i in currentMsg.targetshapes2) {
            if (currentMsg.targetshapes2[i].shapeid == currentShape.prev) {
                previousShape = currentMsg.targetshapes2[i];
                break;
            }
        }
    }
    nextShape = null;
    if (currentShape.next != undefined) {
        for (var i in currentMsg.targetshapes2) {
            if (currentMsg.targetshapes2[i].shapeid == currentShape.next) {
                nextShape = currentMsg.targetshapes2[i];
                break;
            }
        }
    }

    var alldone = false;
    if (currentShape != null) {
        var dx = Math.abs(msg.x - currentShape.posx);
        var dy = Math.abs(msg.y - currentShape.posy);
        var da = Math.abs(msg.angle - currentShape.angle);
        if (currentShape.repeat != undefined) {
            da = da % currentShape.repeat;
        } else {
            da = da % 360;
        }

        isRightAngle = (da < 2 || da > 358);
        if (dx < 3 && dy < 3) {
            if (dy > dx) {
                side = (msg.y - currentShape.posy > 0) ? 2 : 0;
            } else {
                side = (msg.x - currentShape.posx > 0) ? 1 : 3;
            }
            isClose = true;
        }
        if (dx < 0.7 && dy < 0.7 && (da < 2 || da > 358)) {
            currentShape.inPlace = true;
            //check if alldone only when a new block is in place
            alldone = true;
            for (var i in currentMsg.targetshapes2) {
                if (currentMsg.targetshapes2[i].inPlace == undefined) {
                    alldone = false;
                    break;
                }
            }
        }

        if (alldone) {
            actionList[finishAction].msg.shapeIdx = currentShape.shapeid;
            actionList[finishAction].msg.posx = currentShape.posx;
            actionList[finishAction].msg.posy = currentShape.posy;
            actionList[finishAction].msg.angle = currentShape.angle;
            doAction(finishAction);
        } else {
            if (currentShape.inPlace) {
                actionList[moveAction].msg.isright = true;
                actionList[moveAction].msg.shapeIdx = currentShape.shapeid;
                actionList[moveAction].msg.posx = currentShape.posx;
                actionList[moveAction].msg.posy = currentShape.posy;
                actionList[moveAction].msg.angle = currentShape.angle;
            } else {
                actionList[moveAction].msg.isright = false;
            }
            doAction(moveAction);
        }
    }
}

function AE_onChangebackground(flag) {
    if (flag) {
        doAction(correctBkgAction);
    } else {
        doAction(wrongBkgAction);
    }
}
